<?php

/**
 * @file
 * Contains performance check functions.
 */

/**
 * Main performance check class.
 */
abstract class PerformanceCheck {

  // Test data.
  protected $data;

  /**
   * Setup code ran before executing operations (usually used to prepare data for operations to process).
   */
  protected function setup($nruns) {

  }

  /**
   * Operation callback.
   */
  protected function operation(array $test_item) {

  }

  /**
   * Execute function for the operations.
   */
  public function execute($nruns) {
    // Prepare data.
    $this->data = $this->setup($nruns);

    // Time test.
    $start_mt = microtime(TRUE);
    foreach ($this->data as $test_item) {
      $this->operation($test_item);
    }
    $total_time = microtime(TRUE) - $start_mt;
    $result['time']['total'] = round($total_time * 1000, 3);
    $result['time']['op_average'] = round($total_time / count($this->data) * 1000, 3);

    // Memory test.
    $result['memory']['data'] = array();
    $initial_memory = memory_get_usage();
    foreach ($this->data as $index => $test_item) {
      $this->operation($test_item);
      $result['memory']['data'][$index] = memory_get_usage() - $initial_memory;
    }

    $total_memory = 0;
    foreach ($result['memory']['data'] as $op_usage) {
      $total_memory += $op_usage;
    }
    $result['memory']['average'] = $total_memory / count($this->data);

    return $result;
  }

}

/**
 * Example: checking performance of Drupal translation function.
 */
class TranslationPerformanceCheck extends PerformanceCheck {

  /**
   * {@inheritdoc}
   */
  protected function setup($nruns) {
    $lids = db_select('locales_target', 't')->fields('t', array('lid'))->execute()->fetchCol();
    $nstrings = count($lids);

    $to_check = array();
    for ($i = 0; $i < $nruns; $i++) {
      do {
        $rand = rand(0, $nstrings);
        $lid = $lids[$rand];
      } while (isset($to_check[$lid]));

      $to_check[$lid] = $lid;
    }

    if (!empty($to_check)) {
      $query = db_select('locales_source', 's');
      $query->join('locales_target', 't', 't.lid = s.lid');
      $translatables = $query->fields('s', array(
          'lid',
          'source',
          'context'
        ))
        ->fields('t', array('language'))
        ->condition('s.lid', $to_check)
        ->execute()
        ->fetchAllAssoc('lid', PDO::FETCH_ASSOC);
    }
    else {
      $translatables = array();
    }

    return $translatables;
  }

  /**
   * {@inheritdoc}
   */
  protected function operation(array $test_item) {
    t($test_item['source'], array(), array('context' => $test_item['context'], 'langcode' => $test_item['language']));
  }

}
