<?php

/**
 * @file
 * Contains Request monitoring procedures to get execution time for each phase on a page load.
 */

/**
 * Request stage monitoring class.
 */
class ConsoleRequestMonitoring {
  // Request start time.
  private $data;

  /**
   * Initialize.
   */
  public function __construct() {
    $this->data[0] = array(
      'name' => 'init',
      'time' => $_SERVER["REQUEST_TIME_FLOAT"]
    );
  }

  /**
   * Add phase method.
   *
   * @param string $phase_name
   *   The name of the phase.
   */
  public function phase($phase_name) {
    $index = count($this->data);
    $this->data[$index] = array(
      'name' => $phase_name,
      'time' => microtime(TRUE)
    );
  }

  /**
   * Prepare results.
   */
  public function results() {
    $results = array();
    for ($i = 1; $i < count($this->data); $i++) {
      $results[] = array(
        'phase' => $this->data[$i]['name'],
        'duration' => round(($this->data[$i]['time'] - $this->data[$i - 1]['time']) * 1000, 3)
      );
    }

    $total_time = round(($this->data[count($this->data) - 1]['time'] - $this->data[0]['time']) * 1000, 3);

    return array(
      'info' => array(
        'path' => current_path(),
        'request_total' => $total_time
      ),
      'phases' => $results
    );
  }

}

/**
 * API function to add next monitoring phase.
 */
function dcperm($phase, $return_results = FALSE) {
  static $access;
  if (!isset($access)) {
    $ips = variable_get('developer_console_check_ips', array());
    if (in_array(ip_address(), $ips, TRUE)) {
      $access = TRUE;
    }
    else {
      $access = FALSE;
    }
  }
  if ($access) {
    static $monitoring;
    if (!isset($monitoring)) {
      $monitoring = new ConsoleRequestMonitoring();
    }

    $monitoring->phase($phase);

    if ($return_results) {
      return $monitoring->results();
    }
  }
  return FALSE;
}

/**
 * First move all hook calls from this module to the top, except the exit hook.
 */
function developer_console_module_implements_alter(&$implementations, $hook) {
  if (isset($implementations['developer_console'])) {
    $module = 'developer_console';
    if ($hook == 'exit') {
      $group = $implementations[$module];
      unset($implementations[$module]);
      $implementations[$module] = $group;
    }
    else {
      unset($implementations[$module]);
      $new = array($module => FALSE);
      foreach ($implementations as $key => $value) {
        $new[$key] = $value;
      }
      $implementations = $new;
    }
  }
}

/**
 * Standard monitoring hooks.
 */

/**
 * Implements hook_boot().
 */
function developer_console_boot() {
  dcperm('Boot (hook_boot)');
}

/**
 * Implements hook_init().
 */
function developer_console_init() {
  dcperm('After boot (hook_init)');
}

/**
 * Implements hook_page_build().
 */
function developer_console_page_build(&$page) {
  dcperm('Page prepared (hook_page_build)');
}

/**
 * Implements hook_views_pre_build().
 */
function developer_console_views_pre_view($view) {
  dcperm('View ' . $view->name . ' (' . $view->current_display . ') initialized');
}

/**
 * Implements hook_views_post_execute().
 */
function developer_console_views_post_execute($view) {
  dcperm('View ' . $view->name . ' (' . $view->current_display . ') executed');
}

/**
 * Implements hook_views_post_render().
 */
function developer_console_views_post_render($view, &$output, &$cache) {
  dcperm('View ' . $view->name . ' (' . $view->current_display . ') rendered');
}

/**
 * Implements hook_exit().
 */
function developer_console_exit() {
  if ($results = dcperm('Page rendered, output sent to browser (hook_exit)', TRUE)) {
    print '<pre>' . print_r($results, TRUE) . '</pre>';
  }
}
